/***************************************************************************
 *       Copyright (c) 2003,2004 by Marcel Wiesweg                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
  
#include <map>
#include <time.h>
    
#include <vdr/interface.h>
#include <vdr/i18n.h>

#include "menu.h"
#include "txtbitmap.h"
#include "display.h"
#include "setup.h"
#include "txtrecv.h"


#define TXTROOT "/vtx"

#define GET_HUNDREDS(x) ( ( (x) - ((x)%256) ) /256 )
#define GET_TENS(x)  ( (( (x) - ((x)%16) )%256 ) /16 )
#define GET_ONES(x)   ( (x)%16 )

#define GET_HUNDREDS_DECIMAL(x) ( ( (x) - ((x)%100) ) /100 )
#define GET_TENS_DECIMAL(x)  ( (( (x) - ((x)%10) )%100 ) /10 )
#define GET_ONES_DECIMAL(x)   ( (x)%10 )

#define PSEUDO_HEX_TO_DECIMAL(x) ( (GET_HUNDREDS_DECIMAL(x))*256 + (GET_TENS_DECIMAL(x))*16 + (GET_ONES_DECIMAL(x)) )

using namespace std;
   
int Stretch = true;
typedef map<int,int> IntMap;
IntMap channelPageMap;

//static variables
int TeletextBrowser::currentPage=0x100; //Believe it or not, the teletext numbers are somehow hexadecimal
int TeletextBrowser::currentSubPage=0;
tChannelID TeletextBrowser::channel;
int TeletextBrowser::currentChannelNumber=0;
Display::Mode TeletextBrowser::mode=Display::Full;
TeletextBrowser* TeletextBrowser::self=0;


TeletextBrowser::TeletextBrowser(cTxtStatus *txtSt) {
   cursorPos=0;
   txtbitmap=0;
   pageFound=true;
   selectingChannel=false;
   selectingChannelNumber=-1;
   self=this;
   txtStatus=txtSt;
   //if (txtStatus)
    //  txtStatus->ForceReceiving(true);
   suspendedReceiving=false;  
   previousPage=currentPage;
   previousSubPage=currentSubPage;
   pageBeforeNumberInput=currentPage;
}


TeletextBrowser::~TeletextBrowser() {
   if (txtbitmap)
      delete txtbitmap;
   self=0;
   Display::Close();
   /*if (txtStatus) {
      if (suspendedReceiving)
         txtStatus->ForceSuspending(false);
      txtStatus->ForceReceiving(false);
   }*/
}

void TeletextBrowser::Show(void) {
   Display::SetBackground((tColor)ttSetup.configuredClrBackground);
   Display::Open(mode);
   ShowPage();
   ShowPageNumber();
  // Display::DrawRect(15,10,490,506, BLU);
  // DisplayUpdate();
}

void TeletextBrowser::ChannelSwitched(int ChannelNumber) {
   cChannel *chan=Channels.GetByNumber(ChannelNumber);
   tChannelID chid=chan->GetChannelID();
   if (chid==channel)
      return;
   channel=chid;
   
   //store page number of current channel
   IntMap::iterator it;
   //channelPageMap.insert(IntMap::value_type(currentChannelNumber, currentPage));
   channelPageMap[currentChannelNumber] = currentPage;
   currentChannelNumber=ChannelNumber;
   
   currentPage=0x100;
   currentSubPage=0;
      
   //see if last page number on this channel was stored
   it=channelPageMap.find(ChannelNumber);
   if (it != channelPageMap.end()) { //found
      currentPage=(*it).second;
   }
   
   //on the one hand this must work in background mode, when the plugin is not active.
   //on the other hand, if active, the page should be shown.
   //so this self-Pointer.
   if (self) {
      self->ShowPage();
      self->ShowPageNumber();
   }
}


eOSState TeletextBrowser::ProcessKey(eKeys Key) {  
   switch (Key) {
      case k1: SetNumber(1);break;
      case k2: SetNumber(2);break;
      case k3: SetNumber(3);break;
      case k4: SetNumber(4);break;
      case k5: SetNumber(5);break;
      case k6: SetNumber(6);break;
      case k7: SetNumber(7);break;
      case k8: SetNumber(8);break;
      case k9: SetNumber(9);break;
      case k0: 
         //same behavior for 0 as VDR does it with channels
         if ((cursorPos==0)  && (!selectingChannel)) {
            //swap variables
            int tempPage=currentPage;
            int tempSubPage=currentSubPage;
            currentPage=previousPage;
            currentSubPage=previousSubPage;
            previousPage=tempPage;
            previousSubPage=tempSubPage;
            ShowPage();
            ShowPageNumber();
         } else
            SetNumber(0);
         break;
      case kOk: 
         if (selectingChannel) {
            selectingChannel=false;
            if (selectingChannelNumber>0) {
               ChannelSwitched(selectingChannelNumber);
            } else {
               ShowPage();
               ShowPageNumber(); 
            }
         }
         break;        
      case kBack: return osEnd; 
      case kNone: //approx. every second
         //checking if page changed
         if ( pageFound && ttSetup.autoUpdatePage && cursorPos==0 && !selectingChannel && (PageCheckSum() != checkSum) ) {
            ShowPage();
            ShowPageNumber(); 
         }
         //updating clock
         if ( mode != Display::ZoomLower && mode != Display::HalfLower)
            UpdateClock();
         break;
      case kUp:
         if (selectingChannel)
             selectingChannel=false;
         if (cursorPos != 0) {
            //fully revert cursor
            SetNumber(-3);
         }
         ChangePageRelative();
         if (mode==Display::ZoomLower)
            mode=Display::ZoomUpper;
         if (mode==Display::HalfLower)
            mode=Display::HalfUpper;
         ShowPage();
         ShowPageNumber(); 
         UpdateClock();
         break;        
      case kDown:
         if (selectingChannel)
             selectingChannel=false;
         if (cursorPos != 0) {
            //fully reset
            SetNumber(-3);
         }
         ChangePageRelative(true);
         if (mode==Display::ZoomLower)
            mode=Display::ZoomUpper;
         if (mode==Display::HalfLower)
            mode=Display::HalfUpper;
         ShowPage();
         ShowPageNumber();  
         UpdateClock();
         break;       
      case kRight:
         if (selectingChannel)
             selectingChannel=false;
         if (cursorPos != 0) {
            //fully reset
            SetNumber(-3);
         }
         ChangeSubPageRelative();
         if (mode==Display::ZoomLower)
            mode=Display::ZoomUpper;
         if (mode==Display::HalfLower)
            mode=Display::HalfUpper;
         ShowPage();
         ShowPageNumber();  
         UpdateClock();
         break;       
      case kLeft:
         if (selectingChannel)
             selectingChannel=false;
         if (cursorPos != 0) {
            //revert cursor
            SetNumber(-1);
            ShowPageNumber();
            UpdateClock();
            break;
         }
         ChangeSubPageRelative(true);
         if (mode==Display::ZoomLower)
            mode=Display::ZoomUpper;
         if (mode==Display::HalfLower)
            mode=Display::HalfUpper;
         ShowPage();
         ShowPageNumber();   
         UpdateClock();
         break; 
         
              
      case kRed: 
      case kGreen: 
      case kBlue:
      case kYellow:
      //case kUser1:case kUser2:case kUser3:case kUser4:case kUser5:
      //case kUser6:case kUser7:case kUser8:case kUser9:
      case kPlay:case kPause:case kStop: case kRecord:case kFastFwd:case kFastRew:
         if (cursorPos != 0) {
            //fully reset
            SetNumber(-3);
         }
         ExecuteAction(TranslateKey(Key));
         break;             
      default: break;
   }
   return osContinue;
}

void TeletextBrowser::ExecuteAction(eTeletextAction e) {
         switch (e) {
            case Zoom:
               if (selectingChannel)
                  selectingChannel=false;
               if (mode==Display::ZoomUpper)
                  mode=Display::ZoomLower;
               else if (mode==Display::ZoomLower)
                  mode=Display::Full;
               else
                  mode=Display::ZoomUpper;
               Display::CheckMode(mode);
               ShowPage();
               if (mode!=Display::ZoomLower) {
                  ShowPageNumber();
                  UpdateClock();                
               }
               break;
            case HalfPage:
               if (selectingChannel)
                  selectingChannel=false;
               if (mode==Display::HalfUpper)
                  mode=Display::HalfLower;
               else if (mode==Display::HalfLower)
                  mode=Display::Full;
               else
                  mode=Display::HalfUpper;
               Display::CheckMode(mode);
               ShowPage();
               if (mode!=Display::HalfLower) {
                  ShowPageNumber();
                  UpdateClock();
               }
               break;
            case SwitchChannel:
               selectingChannelNumber=0;
               selectingChannel=true;
               ShowAskForChannel();
               break;
            /*case SuspendReceiving:
               if (!txtStatus)
                  break;
               //if (suspendedReceiving)
                //  txtStatus->ForceSuspending(false);
               //else
                //  txtStatus->ForceSuspending(true);
               //suspendedReceiving=(!suspendedReceiving);
               break;*/
            case DarkScreen:
               Display::Close(); //necessary to change color
               if (Display::GetBackground() == clrBlack)
                  Display::SetBackground((tColor)ttSetup.configuredClrBackground);
               else
                  Display::SetBackground(clrBlack);
               Display::Open(mode);
               ShowPage();
               if (mode!=Display::ZoomLower) {
                  ShowPageNumber();
                  UpdateClock();                
               }
               break;
            default:
               //In osdteletext.c, numbers are thought to be decimal, the setup page
               //entries will display them in this way. It is a lot easier to do the
               //conversion to hexadecimal here.
               //This means, we convert the number to what it would be if the string
               //had been parsed with hexadecimal base.
               int pageNr=PSEUDO_HEX_TO_DECIMAL((int)e);
               if (0x100<=pageNr<=0x899) {
                  if (selectingChannel)
                     selectingChannel=false;
                  SetPreviousPage(currentPage, currentSubPage, pageNr);
                  currentPage=pageNr;
                  cursorPos=0;
                  currentSubPage=0;
                  if (mode==Display::ZoomLower)
                     mode=Display::ZoomUpper;
                  if (mode==Display::HalfLower)
                     mode=Display::HalfUpper;
                  ShowPage();
                  ShowPageNumber();
                  UpdateClock();
               }
               break;
         }
}

eTeletextAction TeletextBrowser::TranslateKey(eKeys Key) {
   switch(Key) {
      case kRed:     return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyRed];
      case kGreen:   return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyGreen];
      case kYellow:  return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyYellow];
      case kBlue:    return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyBlue];
      case kPlay:   return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyPlay];
      //case kPause:   return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyPause];
      case kStop:   return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyStop];
      //case kRecord:   return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyRecord];
      case kFastFwd:   return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyFastFwd];
      case kFastRew:   return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyFastRew];
      default: return (eTeletextAction)100; //just to keep gcc quiet
   }
}


void TeletextBrowser::SetNumber(int i) {
   //cursorPos means insertion after, 0<=cursorPos<=2
   if (selectingChannel) {
      selectingChannelNumber = selectingChannelNumber*10+i;  
      ShowAskForChannel();
      return;
   }
   
   //i<0 means revert cursor position
   if (i<0) {
      for (;i<0;i++) {
         switch (cursorPos) {
         case 0:
            return;
         case 1:
            currentPage = currentPage-256*GET_HUNDREDS(currentPage)+256*GET_HUNDREDS(pageBeforeNumberInput);
            break;
         case 2:
            currentPage = currentPage-16*GET_TENS(currentPage)+16*GET_TENS(pageBeforeNumberInput);
            break;
         }
         cursorPos--;
      }
      return;
   }

   
   static int tempPage;
   switch (cursorPos) {
   case 0:
      if (i<1) i=1;
      //accept no 9 when cursorPos==0
      if (i>8) i=8;
      tempPage= currentPage;
      pageBeforeNumberInput = currentPage;
      currentPage = currentPage-256*GET_HUNDREDS(currentPage)+256*i;
      break;
   case 1:
      if (i<0) i=0;
      if (i>9) i=9;
      currentPage = currentPage-16*GET_TENS(currentPage)+16*i;
      break;
   case 2:
      if (i<0) i=0;
      if (i>9) i=9;
      currentPage = currentPage-GET_ONES(currentPage)+i;
      pageBeforeNumberInput = currentPage;
      SetPreviousPage(tempPage, currentSubPage, currentPage);
      break;
   }
   pageFound=true; //so that "page ... not found" is not displayed, but e.g. 1**-00
   if (++cursorPos>2) {
      cursorPos=0;
      FindFirstSubPage(0);
      if (mode==Display::ZoomLower)
         mode=Display::ZoomUpper;
      if (mode==Display::HalfLower)
         mode=Display::HalfUpper;
      ShowPage();
   }
   ShowPageNumber();
   UpdateClock();
}

//returns whether x, when written in hexadecimal form,
//will only contain the digits 0...9 and not A...F
//in the first three digits.
static inline bool onlyDecimalDigits(int x) {
   return ((  x      & 0xE) < 0xA) &&
          (( (x>>4)  & 0xE) < 0xA) &&
          (( (x>>8)  & 0xE) < 0xA);
}

//after 199 comes 1A0, but if these pages exist, they contain no useful data, so filter them out
int TeletextBrowser::nextValidPageNumber(int start, bool direction) {
   do {
      if (direction)
         start++;
      else
         start--;
   } while (!onlyDecimalDigits(start));
   return start;
}

void TeletextBrowser::ChangePageRelative(bool back)
{
   int oldpage = currentPage;
   int oldSubPage = currentSubPage;

   do  {
      /*if (back)
         currentPage--;
      else
         currentPage++;*/
      currentPage=nextValidPageNumber(currentPage, !back);
      if (currentPage>0x899) currentPage=0x100;
      if (currentPage<0x100) currentPage=0x899;
      // sub page is always 0 if you change the page
      FindFirstSubPage(0);
      if (CheckPage()) {
         SetPreviousPage(oldpage, oldSubPage, currentPage);
         return;
      }
   } while (currentPage != oldpage);

   return;
}

void TeletextBrowser::ChangeSubPageRelative(bool back)
{
   int oldsubpage = currentSubPage;

   do  {
      /*if (back)
         currentSubPage--;
      else
         currentSubPage++;*/
      currentSubPage=nextValidPageNumber(currentSubPage, !back);
      if (currentSubPage > 0x99) currentSubPage=0;
      if (currentSubPage < 0) currentSubPage=0x99;

      if (CheckPage())
         return;
   } while (currentSubPage != oldsubpage);

   return;
}

void TeletextBrowser::FindFirstSubPage(int startWith) {
   int oldsubpage = currentSubPage;

   do  {
      if (CheckPage())
         return;
      //currentSubPage++;
      currentSubPage=nextValidPageNumber(currentSubPage);
      
      if (currentSubPage > 0x99) currentSubPage=0;
      if (currentSubPage < 0) currentSubPage=0x99;

   } while (currentSubPage != oldsubpage);
   
}

bool TeletextBrowser::CheckPage()
{
   StorageHandle fd;
   
   Storage *s=Storage::instance();
   if (!(fd=s->openForReading(PageID(channel, currentPage, currentSubPage), false)) )
      return false;

   s->close(fd);
   return true;
}

//sets the previousPage variables if and only if new page is different from old page
void TeletextBrowser::SetPreviousPage(int oldPage, int oldSubPage, int newPage)  {
   if (oldPage != newPage) {
      previousPage=oldPage;
      previousSubPage=oldSubPage;
   }
}




void TeletextBrowser::ShowPage() {
   if ((pageFound=DecodePage()) && txtbitmap) {
      txtbitmap->Display(mode, 0/*cOsd::LineHeight()*/);
      if (ttSetup.autoUpdatePage)
         checkSum=PageCheckSum();
   }
}

void TeletextBrowser::ShowPageNumber() {
   if (pageFound) {
      char str[8];
      sprintf(str, "%3x-%02x", currentPage, currentSubPage);
      if (cursorPos>0) {
         str[2]='*';
         if (cursorPos==1)
            str[1]='*';
      }
      Display::ClearMessage();
      Display::Message(str);
      Display::Update();
   } else {
      char str[80];
      snprintf(str,80, "%s %3x-%02x %s",tr("Page"),currentPage, currentSubPage,tr("not found"));      
      Display::ClearMessage();
      Display::Message(str);
      Display::Update();
   }
}

void TeletextBrowser::ShowAskForChannel() {
   if (selectingChannel) {
      char *str;
      if (selectingChannelNumber>0)
         asprintf(&str,"%s%d", tr("Channel (press OK): "), selectingChannelNumber);
      else
         asprintf(&str,"%s", tr("Channel (press OK): ") );
      Display::ClearMessage();
      Display::Message(str);
      Display::Update();
      free(str);
   }
}

//this is taken and adapted from the teletext plugin since it uses its data
bool TeletextBrowser::DecodePage() {
   // Load the page and decodes it
   //char str[80];
   char lzeichen;
   char letztes_zeichen;
   int x,y;
   // Variablen zur Paramterspeicherung
   bool flashing=false;
   bool highdouble=false;
   bool conceal=false;
   bool sepgraph=false;
   bool hold_gfx=false;
   bool graph_mode=false;
   bool skipnextline=false;
   
   
   
   char cache[100];
   StorageHandle fd;
   // Take a look if there is a xxx-00 page
   Storage *s=Storage::instance();
   if (currentSubPage==0) {
      if ( !(fd=s->openForReading(PageID(channel, currentPage,currentSubPage), false)) ) {
         // There is no subpage 0 so look if there is subpage 1
         currentSubPage++;
         // Generate file string
      } else {
         // yes file exists
         s->close(fd);
      }
   }
   
   if ( (fd=s->openForReading(PageID(channel, currentPage, currentSubPage), true)) ) 
   {
      // generate a basis bitmap object
      if (txtbitmap)
         delete txtbitmap;
   
      txtbitmap = new cTxtBitmap(486,506); 
      
      s->read(cache,9,fd); //irgendwas davor
      s->read(cache,1,fd); //Language
      //txtbitmap->SetLanguage((unsigned int)cache[0]);
      s->read(cache,2,fd); //irgendwas wieder
      for (y=0;y<24;y++) {
      
         flashing=false;
         highdouble=false;
         conceal=false;
         sepgraph=false;
         hold_gfx=false;
         graph_mode=false;
         skipnextline=false;
         letztes_zeichen=' ';
      
         txtbitmap->SetFGColor((Display::TeletextColor)7);
         txtbitmap->SetBGColor((Display::TeletextColor)0);
         
         
         
         for (x=0;x<40;x++)
         {
            s->read(&lzeichen, 1, fd);
            lzeichen=lzeichen & 0x7F; //Parity Bit ist uninteressant!
            
            if ((y==0)&&(x<8)) { // Die Daten sind uninteressant zur Anzeige!
               lzeichen=0x20;
            }
      
         
         
         
            // conceal = verdeckt!!!
         
            switch (lzeichen) {
            case 0x00 ... 0x07 :
                        conceal=false;
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        graph_mode=false;
                        txtbitmap->SetFGColor((Display::TeletextColor)((int)lzeichen));
                        break;
            case 0x08 : // Blinken einschalten (flashing)
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        flashing=true;
                        break;
            case 0x09 : // Blinken ausschalten (steady)
                        flashing=false;
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
            case 0x0A : // end box (nicht benutzt)
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x0B : // start box (nicht benutzt)
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x0C : // normal high
                        highdouble=false;
                        txtbitmap->SetChar(x,y,' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x0D : // double high
                        for (int frei=1;frei<40;frei++)
                           txtbitmap->SetChar(frei,y+1,' ',graph_mode,highdouble,sepgraph);
                        txtbitmap->SetChar(x,y,' ',graph_mode,highdouble,sepgraph);
                        highdouble=true;
                        skipnextline=true;
                        break;
            case 0x0E ... 0x0F : // keine Funktion
                        break;
            case 0x10 ... 0x17  :
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        conceal=false;
                        graph_mode=true;
                        txtbitmap->SetFGColor((Display::TeletextColor)(((int)lzeichen)-0x10));
                        break;
            case 0x18 : // verborgen
                        conceal=true;
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x19 : // contigouous graphics
                        sepgraph=false;
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x1A : // separated grphics
                        sepgraph=true;
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x1B : // ESC
                        break;
            case 0x1C : // black background
                        txtbitmap->SetBGColor((Display::TeletextColor)0);
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x1D : // new background
                        txtbitmap->ExchangeColor();
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x1E : // hold graphics
                        hold_gfx=true;
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        break;
            case 0x1F : // release grphics
                        txtbitmap->SetChar(x,y,hold_gfx?letztes_zeichen:' ',graph_mode,highdouble,sepgraph);
                        hold_gfx=false;
                        break;
            default:
                  if (graph_mode==true) letztes_zeichen=lzeichen;
                     txtbitmap->SetChar(x,y,lzeichen,graph_mode,highdouble,sepgraph);
            }
         }
         if (skipnextline==true) {
               y++;
      //Rolf Ahrenberg
      //         skipnextline=false;
         }
      }
      s->close(fd);
   } else {
      // page doesn't exist
      currentSubPage--;
      return false;
   }
   return true;
}



int TeletextBrowser::PageCheckSum() {
   int retSum=0;
   StorageHandle fd;
   
   FindFirstSubPage(currentSubPage);
   
   Storage *s=Storage::instance();
   if ((fd=s->openForReading(PageID(channel, currentPage, currentSubPage), false)) ) {
      uchar cache[960];
      s->read(cache, 12, fd); //skip
      s->read(cache, sizeof(cache), fd);
      s->close(fd);
      memset(cache+12, 0, 8); //it seems that there the clock is transmitted, ignore changes
      for (uint i=0;i<sizeof(cache); i++)
         retSum+=cache[i];
   }
   return retSum;
}

void TeletextBrowser::UpdateClock() {
   if ( ttSetup.showClock && txtbitmap)
      txtbitmap->DisplayClock();
}


ChannelStatus::ChannelStatus() 
{
}


void ChannelStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber) {
   if (Device->IsPrimaryDevice() && ChannelNumber>0) 
      TeletextBrowser::ChannelSwitched(ChannelNumber);
}

TeletextSetup ttSetup;

TeletextSetup::TeletextSetup() {
   configuredClrBackground=clrGray50;
   
   //init key bindings with default values
   for (int i=0;i<10;i++)
      mapKeyToAction[0]=(eTeletextAction)0;
   mapKeyToAction[3]=Zoom;
   mapKeyToAction[2]=HalfPage;   
   mapKeyToAction[0]=SwitchChannel;
   
   showClock=true;
   suspendReceiving=false;
   autoUpdatePage=true;
   //OSDHeight+width default values given in Start()
}

